namespace Mail
{
	using System;
	using System.Text;
	using System.Text.RegularExpressions;
	using System.Collections;
	using System.Diagnostics;

	using Mail.Providers;
	/// <summary>
	///   ,   .	
	/// </summary>
	public class Message : Common, IDisposable
	{
		string _body;
		//   .
		BodyTypes _body_type = BodyTypes.Unknown;
		//  .
		AttachDescriptor [] _attaches = null;

		/// <summary>
		///  .
		/// </summary>
		public uint Index;


		/// <summary>
		///    .
		/// </summary>
		/// <param name="parent">  .</param>
		/// <param name="message"> ,   .</param>
		/// <param name="index"> .</param>
		public Message(Provider parent, string message, uint index) : base(parent, message)
		{
			//     ,     ArgumentOutOfRangeException
			if (index < 1)
			{
				throw new ArgumentOutOfRangeException("index");
			}

			Index = index;
			ParseContentType();
		}

		/// <summary>
		///  .
		/// </summary>
		public AttachDescriptor [] Attachments
		{
			get
			{
				if (_attaches == null)
				{
					ArrayList al = new ArrayList();
                    GetAllAttachments(ChildMimes, al);
					_attaches = (AttachDescriptor [])al.ToArray(typeof(AttachDescriptor));
				}

				return _attaches;
			}
		}

		/// <summary>
		///   .
		/// </summary>
		/// <param name="mimes"></param>
		void GetAllAttachments(Mime [] mimes, ArrayList al)
		{
			foreach (Mime m in mimes)
			{
				if (m.ChildMimes.Length == 0)
				{
					if (m._attach != null) al.Add(m._attach);
				} 
				else
				{
					GetAllAttachments(m.ChildMimes, al);
				}
			}
		}
	
		//   .
		void ParseContentType()
		{
			if (ContentType == null)
			{
                throw new ParseException("   (Content-Type )");
			}

			string type;
			int i = ContentType.IndexOf(";");
			if (i < 0)
			{
				type = ContentType;
			}
			else
			{
                type = ContentType.Substring(0, i);
			}
			//          .
			switch(type)
			{
				case "multipart/mixed":
					MultipartMixed();
					break;

				case "multipart/alternative":
					MultipartMixed();
					break;

				case "multipart/related":
					MultipartMixed();
					break;

				case "text/html":
					_body = _message;
					_body_type = BodyTypes.HTML;
					break;

				case "text/plain":
					_body = _message;
					_body_type = BodyTypes.Text;
					break;
			}
		}

		/// <summary>
        ///     email.		
		/// </summary>
		/// <param name="mail">  </param>
		/// <returns>  [mail@host.com], [mail@localhost]  [host@123.123.123.123]</returns>
		public string ExtractEmailFromAddress(string mail)
		{
			// mail@(ip)|(host)
			Regex ex = new Regex(@"([a-zA-Z0-9_\-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([a-zA-Z0-9\-]+\.?)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)");
			Match m = ex.Match(mail);
			if (!m.Success) 
			{
				throw new ParseException("email  .");
			}

			return m.ToString();
		}

		public void Dispose()
		{
			foreach(AttachDescriptor ad in Attachments)
			{
                ad.Close();
			}            

			_attaches = null;
			_body = null;
			_message = null;
			Headers = null;
			ChildMimes = null;

			GC.SuppressFinalize(this);
		}
		/// <summary>
		///   ToString.    .
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			StringBuilder sb = new StringBuilder();

			sb.AppendFormat("Size: {0}b\r\n", GetSize());
			sb.AppendFormat("From: '{0}', email: '{1}'\r\n", From, FromEmail);
			sb.AppendFormat("Subject: '{0}'\r\n", Subject);

			if (Attachments.Length > 0)
			{
				sb.Append("Attachments:\r\n");

				foreach(AttachDescriptor ad in Attachments)
				{
					sb.Append("\tName: " + ad.Name + " Size: " + ad.Size + "\r\n");
				}
			}

			return sb.ToString();
		}
		// 
		/// <summary>
		///    .
		/// </summary>
		public enum BodyTypes
		{
			Unknown,
            HTML,
			Text
		}

		/// <summary>
		///      MIME.
		/// </summary>
		/// <param name="type">  .</param>
		/// <param name="mimes">MIME</param>
		/// <returns></returns>
		string GetBodyFromMime(BodyTypes type, Mime [] mimes)
		{
			foreach (Mime m in mimes)
			{
				if (m.ChildMimes.Length == 0)
				{
					switch(type)
					{
						case BodyTypes.HTML:
							if (m.ContentType.IndexOf("text/html") > -1)
							{
								return m.PreparedBody();
							}
							break;

						case BodyTypes.Text:
							if (m.ContentType.IndexOf("text/plain") > -1)
							{
								return m.PreparedBody();
							}
							break;
					}
				} 
				else
				{
					string r = GetBodyFromMime(type, m.ChildMimes);
					if (r != "") return r;
				}
			}

			return "";
		}

		/// <summary>
		///     .
		/// </summary>
		/// <param name="type">  .</param>
		/// <returns></returns>
		public string GetBody(BodyTypes type)
		{
			if (_body_type == BodyTypes.Unknown)
				return GetBodyFromMime(type, ChildMimes);
			else
				return PreparedString(_body);

		}

		/// <summary>
		///   .
		/// </summary>
		/// <returns></returns>
		public string Text
		{
			get
			{
				string text;
				if (_body_type == BodyTypes.Unknown)
				{
					text = GetBodyFromMime(BodyTypes.Text, ChildMimes);
					if (text == null || text.Trim() == "")
						text = Utils.ExtractTextFromHtml(GetBodyFromMime(BodyTypes.HTML, ChildMimes));
				}
				else
				{
					text = PreparedString(_body);
                    if (_body_type == BodyTypes.HTML) 
						text = Utils.ExtractTextFromHtml(text);
				}

				return text.Trim();
			}
		}

		#region Common headers
        /// <summary>
        /// .
        /// </summary>
		public string Organization
		{
			get
			{
				return (string)Headers["organization"];
			}
		}

		/// <summary>
		///  .
		/// </summary>		
		public string CC
		{
			get
			{
				return (string)Headers["cc"];
			}
		}

		/// <summary>
		///  .
		/// </summary>
		public string Date
		{
			get
			{
				return (string)Headers["date"];
			}
		}

		/// <summary>
		///  .
		/// </summary>
		public string ReturnPath
		{
			get
			{
				return (string)Headers["return-path"];
			}
		}

		/// <summary>
		///  .
		/// </summary>		
		public string From
		{
			get
			{
                return (string)Headers["from"];
			}
		}
        /// <summary>
        ///  .
        /// </summary>
		public string FromEmail
		{
			get
			{
				return ExtractEmailFromAddress((string)Headers["from"]);
			}
		}
        /// <summary>
        /// 
        /// </summary>
		public string To
		{
			get
			{
				return (string)Headers["to"];
			}
		}
        /// <summary>
        /// 
        /// </summary>
		public string Subject
		{
			get
			{
				return Utils.WordDecoder((string)Headers["subject"]);
			}
		}
        /// <summary>
        /// 
        /// </summary>
		public string ReplyTo
		{
			get
			{
				return (string)Headers["reply-to"];
			}
		}
		#endregion
	}
}